@@ -122,10 +122,10 @@ gst_pixbufscale_base_init (gpointer g_class)
       "Wim Taymans <wim.taymans@chello.be>, "
       "Renato Filho <renato.filho@indt.org.br>");
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_pixbufscale_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_pixbufscale_sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_pixbufscale_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_pixbufscale_sink_template));
 }
 
 static void
@@ -144,7 +144,7 @@ gst_pixbufscale_class_init (GstPixbufScaleClass * klass)
       ARG_METHOD,
       g_param_spec_enum ("method", "method", "method",
           GST_TYPE_PIXBUFSCALE_METHOD, GST_PIXBUFSCALE_BILINEAR,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   trans_class->transform_caps =
       GST_DEBUG_FUNCPTR (gst_pixbufscale_transform_caps);
