@@ -1,7 +1,6 @@
 /* GStreamer
  * Copyright (C) <1999> Erik Walthinsen <omega@cse.ogi.edu>
  * Copyright (C) <2006> Tim-Philipp Müller <tim centricular net>
- * Copyright (C) <2012> Ralph Giles <giles@mozilla.com>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -71,17 +70,12 @@ enum
 
 static GstElementClass *parent_class = NULL;
 
-#ifdef SHOUT_FORMAT_WEBM
-#define WEBM_CAPS "; video/webm"
-#else
-#define WEBM_CAPS ""
-#endif
 static GstStaticPadTemplate sink_template = GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS ("application/ogg; "
-        "audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]" WEBM_CAPS));
-
+        "audio/mpeg, mpegversion = (int) 1, layer = (int) [ 1, 3 ]")
+    );
 static void gst_shout2send_class_init (GstShout2sendClass * klass);
 static void gst_shout2send_base_init (GstShout2sendClass * klass);
 static void gst_shout2send_init (GstShout2send * shout2send);
@@ -166,7 +160,8 @@ gst_shout2send_base_init (GstShout2sendClass * klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class, &sink_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&sink_template));
   gst_element_class_set_details_simple (element_class, "Icecast network sink",
       "Sink/Network", "Sends data to an icecast server",
       "Wim Taymans <wim.taymans@chello.be>, "
@@ -192,52 +187,50 @@ gst_shout2send_class_init (GstShout2sendClass * klass)
   gobject_class->finalize = (GObjectFinalizeFunc) gst_shout2send_finalize;
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_IP,
-      g_param_spec_string ("ip", "ip", "ip", DEFAULT_IP,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+      g_param_spec_string ("ip", "ip", "ip", DEFAULT_IP, G_PARAM_READWRITE));
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PORT,
       g_param_spec_int ("port", "port", "port", 1, G_MAXUSHORT, DEFAULT_PORT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PASSWORD,
       g_param_spec_string ("password", "password", "password", DEFAULT_PASSWORD,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_USERNAME,
       g_param_spec_string ("username", "username", "username", DEFAULT_USERNAME,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   /* metadata */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PUBLIC,
       g_param_spec_boolean ("public", "public",
           "If the stream should be listed on the server's stream directory",
-          DEFAULT_PUBLIC, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_PUBLIC, G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_STREAMNAME,
       g_param_spec_string ("streamname", "streamname", "name of the stream",
-          DEFAULT_STREAMNAME, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_STREAMNAME, G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DESCRIPTION,
       g_param_spec_string ("description", "description", "description",
-          DEFAULT_DESCRIPTION, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_DESCRIPTION, G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_GENRE,
       g_param_spec_string ("genre", "genre", "genre", DEFAULT_GENRE,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_PROTOCOL,
       g_param_spec_enum ("protocol", "protocol", "Connection Protocol to use",
-          GST_TYPE_SHOUT_PROTOCOL, DEFAULT_PROTOCOL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          GST_TYPE_SHOUT_PROTOCOL, DEFAULT_PROTOCOL, G_PARAM_READWRITE));
 
 
   /* icecast only */
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_MOUNT,
       g_param_spec_string ("mount", "mount", "mount", DEFAULT_MOUNT,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_URL,
       g_param_spec_string ("url", "url", "url", DEFAULT_URL,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 
   /* signals */
   gst_shout2send_signals[SIGNAL_CONNECTION_PROBLEM] =
@@ -543,14 +536,9 @@ set_failed:
 static gboolean
 gst_shout2send_connect (GstShout2send * sink)
 {
-  const char *format =
+  GST_DEBUG_OBJECT (sink, "Connection format is: %s",
       (sink->audio_format == SHOUT_FORMAT_VORBIS) ? "vorbis" :
-      ((sink->audio_format == SHOUT_FORMAT_MP3) ? "mp3" : "unknown");
-#ifdef SHOUT_FORMAT_WEBM
-  if (sink->audio_format == SHOUT_FORMAT_WEBM)
-    format = "webm";
-#endif
-  GST_DEBUG_OBJECT (sink, "Connection format is: %s", format);
+      ((sink->audio_format == SHOUT_FORMAT_MP3) ? "mp3" : "unknown"));
 
   if (shout_set_format (sink->conn, sink->audio_format) != SHOUTERR_SUCCESS)
     goto could_not_set_format;
@@ -820,10 +808,6 @@ gst_shout2send_setcaps (GstPad * pad, GstCaps * caps)
     shout2send->audio_format = SHOUT_FORMAT_MP3;
   } else if (!strcmp (mimetype, "application/ogg")) {
     shout2send->audio_format = SHOUT_FORMAT_VORBIS;
-#ifdef SHOUT_FORMAT_WEBM
-  } else if (!strcmp (mimetype, "video/webm")) {
-    shout2send->audio_format = SHOUT_FORMAT_WEBM;
-#endif
   } else {
     ret = FALSE;
   }
