@@ -75,7 +75,8 @@ gst_auto_video_src_base_init (gpointer klass)
 {
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (eklass, &src_template);
+  gst_element_class_add_pad_template (eklass,
+      gst_static_pad_template_get (&src_template));
   gst_element_class_set_details_simple (eklass, "Auto video source",
       "Source/Video",
       "Wrapper video source for automatically detected video source",
@@ -90,11 +91,13 @@ gst_auto_video_src_class_init (GstAutoVideoSrcClass * klass)
   GstElementClass *eklass = GST_ELEMENT_CLASS (klass);
 
   gobject_class = G_OBJECT_CLASS (klass);
-  gobject_class->dispose = (GObjectFinalizeFunc) gst_auto_video_src_dispose;
-  gobject_class->set_property = gst_auto_video_src_set_property;
-  gobject_class->get_property = gst_auto_video_src_get_property;
-
+  gobject_class->dispose =
+      (GObjectFinalizeFunc) GST_DEBUG_FUNCPTR (gst_auto_video_src_dispose);
   eklass->change_state = GST_DEBUG_FUNCPTR (gst_auto_video_src_change_state);
+  gobject_class->set_property =
+      GST_DEBUG_FUNCPTR (gst_auto_video_src_set_property);
+  gobject_class->get_property =
+      GST_DEBUG_FUNCPTR (gst_auto_video_src_get_property);
 
   /**
    * GstAutoVideoSrc:filter-caps
@@ -110,7 +113,7 @@ gst_auto_video_src_class_init (GstAutoVideoSrcClass * klass)
   g_object_class_install_property (gobject_class, PROP_CAPS,
       g_param_spec_boxed ("filter-caps", "Filter caps",
           "Filter src candidates using these caps.", GST_TYPE_CAPS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
 }
 
 static void
@@ -132,8 +135,6 @@ gst_auto_video_src_clear_kid (GstAutoVideoSrc * src)
     gst_element_set_state (src->kid, GST_STATE_NULL);
     gst_bin_remove (GST_BIN (src), src->kid);
     src->kid = NULL;
-    /* Don't loose SOURCE flag */
-    GST_OBJECT_FLAG_SET (src, GST_ELEMENT_IS_SOURCE);
   }
 }
 
@@ -175,7 +176,7 @@ gst_auto_video_src_init (GstAutoVideoSrc * src, GstAutoVideoSrcClass * g_class)
   src->filter_caps = gst_static_caps_get (&raw_caps);
 
   /* mark as source */
-  GST_OBJECT_FLAG_SET (src, GST_ELEMENT_IS_SOURCE);
+  GST_OBJECT_FLAG_UNSET (src, GST_ELEMENT_IS_SINK);
 }
 
 static gboolean
