@@ -23,6 +23,7 @@
 #define __GST_MATROSKA_DEMUX_H__
 
 #include <gst/gst.h>
+#include <gst/base/gstadapter.h>
 
 #include "ebml-read.h"
 #include "matroska-ids.h"
@@ -43,7 +44,8 @@ G_BEGIN_DECLS
 typedef enum {
   GST_MATROSKA_DEMUX_STATE_START,
   GST_MATROSKA_DEMUX_STATE_HEADER,
-  GST_MATROSKA_DEMUX_STATE_DATA
+  GST_MATROSKA_DEMUX_STATE_DATA,
+  GST_MATROSKA_DEMUX_STATE_SEEK
 } GstMatroskaDemuxState;
 
 typedef struct _GstMatroskaDemux {
@@ -69,8 +71,10 @@ typedef struct _GstMatroskaDemux {
   gint64                   created;
 
   /* state */
+  gboolean                 streaming;
   GstMatroskaDemuxState    state;
   guint                    level_up;
+  guint64                  seek_block;
 
   /* did we parse cues/tracks/segmentinfo already? */
   gboolean                 index_parsed;
@@ -91,11 +95,30 @@ typedef struct _GstMatroskaDemux {
   /* keeping track of playback position */
   GstSegment               segment;
   gboolean                 segment_running;
-  gint64                   duration;
+  GstClockTime             last_stop_end;
 
   GstEvent                *close_segment;
   GstEvent                *new_segment;
   GstTagList              *global_tags;
+
+  /* push based mode usual suspects */
+  guint64                  offset;
+  GstAdapter              *adapter;
+  /* some state saving */
+  GstClockTime             cluster_time;
+  guint64                  cluster_offset;
+  /* index stuff */
+  gboolean                 seekable;
+  gboolean                 building_index;
+  guint64                  index_offset;
+  GstEvent                *seek_event;
+  gboolean                 need_newsegment;
+
+  /* reverse playback */
+  GArray                  *seek_index;
+  gint                     seek_entry;
+  gint64                   from_offset;
+  gint64                   to_offset;
 } GstMatroskaDemux;
 
 typedef struct _GstMatroskaDemuxClass {
