@@ -63,13 +63,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtpxqtdepay_debug);
 #define GST_CAT_DEFAULT (rtpxqtdepay_debug)
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_xqtdepay_details =
-GST_ELEMENT_DETAILS ("RTP packet depayloader",
-    "Codec/Depayloader/Network",
-    "Extracts Quicktime audio/video from RTP packets",
-    "Wim Taymans <wim@fluendo.com>");
-
 /* RtpXQTDepay signals and args */
 enum
 {
@@ -108,11 +101,6 @@ static gboolean gst_rtp_xqt_depay_setcaps (GstBaseRTPDepayload * depayload,
 static GstBuffer *gst_rtp_xqt_depay_process (GstBaseRTPDepayload * depayload,
     GstBuffer * buf);
 
-static void gst_rtp_xqt_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec);
-static void gst_rtp_xqt_depay_get_property (GObject * object, guint prop_id,
-    GValue * value, GParamSpec * pspec);
-
 static GstStateChangeReturn gst_rtp_xqt_depay_change_state (GstElement *
     element, GstStateChange transition);
 
@@ -126,7 +114,10 @@ gst_rtp_xqt_depay_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_xqt_depay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_xqtdepay_details);
+  gst_element_class_set_details_simple (element_class, "RTP packet depayloader",
+      "Codec/Depayloader/Network",
+      "Extracts Quicktime audio/video from RTP packets",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 static void
@@ -144,9 +135,6 @@ gst_rtp_xqt_depay_class_init (GstRtpXQTDepayClass * klass)
 
   gobject_class->finalize = gst_rtp_xqt_depay_finalize;
 
-  gobject_class->set_property = gst_rtp_xqt_depay_set_property;
-  gobject_class->get_property = gst_rtp_xqt_depay_get_property;
-
   gstelement_class->change_state = gst_rtp_xqt_depay_change_state;
 
   gstbasertpdepayload_class->set_caps = gst_rtp_xqt_depay_setcaps;
@@ -253,11 +241,8 @@ static gboolean
 gst_rtp_xqt_depay_setcaps (GstBaseRTPDepayload * depayload, GstCaps * caps)
 {
   GstStructure *structure;
-  GstRtpXQTDepay *rtpxqtdepay;
   gint clock_rate = 90000;      /* default */
 
-  rtpxqtdepay = GST_RTP_XQT_DEPAY (depayload);
-
   structure = gst_caps_get_structure (caps, 0);
 
   gst_structure_get_int (structure, "clock-rate", &clock_rate);
@@ -676,36 +661,6 @@ unknown_format:
   }
 }
 
-static void
-gst_rtp_xqt_depay_set_property (GObject * object, guint prop_id,
-    const GValue * value, GParamSpec * pspec)
-{
-  GstRtpXQTDepay *rtpxqtdepay;
-
-  rtpxqtdepay = GST_RTP_XQT_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
-static void
-gst_rtp_xqt_depay_get_property (GObject * object, guint prop_id, GValue * value,
-    GParamSpec * pspec)
-{
-  GstRtpXQTDepay *rtpxqtdepay;
-
-  rtpxqtdepay = GST_RTP_XQT_DEPAY (object);
-
-  switch (prop_id) {
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-      break;
-  }
-}
-
 static GstStateChangeReturn
 gst_rtp_xqt_depay_change_state (GstElement * element, GstStateChange transition)
 {
@@ -736,10 +691,3 @@ gst_rtp_xqt_depay_change_state (GstElement * element, GstStateChange transition)
   }
   return ret;
 }
-
-gboolean
-gst_rtp_xqt_depay_plugin_init (GstPlugin * plugin)
-{
-  return gst_element_register (plugin, "rtpxqtdepay",
-      GST_RANK_MARGINAL, GST_TYPE_RTP_XQT_DEPAY);
-}
