@@ -345,14 +345,6 @@ enum
   PROP_MODE_A_ONLY
 };
 
-/* elementfactory information */
-static const GstElementDetails gst_rtp_h263pay_details =
-GST_ELEMENT_DETAILS ("RTP H263 packet payloader",
-    "Codec/Payloader/Network",
-    "Payload-encodes H263 video in RTP packets (RFC 2190)",
-    "Neil Stratford <neils@vipadia.com>"
-    "Dejan Sakelsak <dejan.sakelsak@marand.si>");
-
 static GstStaticPadTemplate gst_rtp_h263_pay_sink_template =
 GST_STATIC_PAD_TEMPLATE ("sink",
     GST_PAD_SINK,
@@ -375,9 +367,6 @@ static GstStaticPadTemplate gst_rtp_h263_pay_src_template =
         "clock-rate = (int) 90000, " "encoding-name = (string) \"H263\"")
     );
 
-static void gst_rtp_h263_pay_class_init (GstRtpH263PayClass * klass);
-static void gst_rtp_h263_pay_base_init (GstRtpH263PayClass * klass);
-static void gst_rtp_h263_pay_init (GstRtpH263Pay * rtph263pay);
 static void gst_rtp_h263_pay_finalize (GObject * object);
 
 static gboolean gst_rtp_h263_pay_setcaps (GstBaseRTPPayload * payload,
@@ -389,8 +378,8 @@ static void gst_rtp_h263_pay_get_property (GObject * object, guint prop_id,
 static GstFlowReturn gst_rtp_h263_pay_handle_buffer (GstBaseRTPPayload *
     payload, GstBuffer * buffer);
 
-static GstRtpH263PayBoundry *gst_rtp_h263_pay_boundry_new (guint8 * start,
-    guint8 * end, guint8 sbit, guint8 ebit);
+static void gst_rtp_h263_pay_boundry_init (GstRtpH263PayBoundry * boundry,
+    guint8 * start, guint8 * end, guint8 sbit, guint8 ebit);
 static GstRtpH263PayGob *gst_rtp_h263_pay_gob_new (GstRtpH263PayBoundry *
     boundry, guint gobn);
 static GstRtpH263PayMB *gst_rtp_h263_pay_mb_new (GstRtpH263PayBoundry * boundry,
@@ -400,42 +389,16 @@ static GstRtpH263PayPackage *gst_rtp_h263_pay_package_new (guint8 * start,
     guint8 * end, guint length, guint8 sbit, guint8 ebit, GstBuffer * outbuf,
     gboolean marker);
 
-static void gst_rtp_h263_pay_boundry_destroy (GstRtpH263PayBoundry * boundry);
 static void gst_rtp_h263_pay_mb_destroy (GstRtpH263PayMB * mb);
 static void gst_rtp_h263_pay_gob_destroy (GstRtpH263PayGob * gob, guint ind);
 static void gst_rtp_h263_pay_context_destroy (GstRtpH263PayContext * context,
     guint ind);
 static void gst_rtp_h263_pay_package_destroy (GstRtpH263PayPackage * pack);
 
-static GstBaseRTPPayloadClass *parent_class = NULL;
+GST_BOILERPLATE (GstRtpH263Pay, gst_rtp_h263_pay, GstBaseRTPPayload,
+    GST_TYPE_BASE_RTP_PAYLOAD)
 
-static GType
-gst_rtp_h263_pay_get_type (void)
-{
-  static GType rtph263pay_type = 0;
-
-  if (!rtph263pay_type) {
-    static const GTypeInfo rtph263pay_info = {
-      sizeof (GstRtpH263PayClass),
-      (GBaseInitFunc) gst_rtp_h263_pay_base_init,
-      NULL,
-      (GClassInitFunc) gst_rtp_h263_pay_class_init,
-      NULL,
-      NULL,
-      sizeof (GstRtpH263Pay),
-      0,
-      (GInstanceInitFunc) gst_rtp_h263_pay_init,
-    };
-
-    rtph263pay_type =
-        g_type_register_static (GST_TYPE_BASE_RTP_PAYLOAD, "GstRtpH263Pay",
-        &rtph263pay_info, 0);
-  }
-  return rtph263pay_type;
-}
-
-static void
-gst_rtp_h263_pay_base_init (GstRtpH263PayClass * klass)
+     static void gst_rtp_h263_pay_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
@@ -444,7 +407,11 @@ gst_rtp_h263_pay_base_init (GstRtpH263PayClass * klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_h263_pay_sink_template));
 
-  gst_element_class_set_details (element_class, &gst_rtp_h263pay_details);
+  gst_element_class_set_details_simple (element_class,
+      "RTP H263 packet payloader", "Codec/Payloader/Network",
+      "Payload-encodes H263 video in RTP packets (RFC 2190)",
+      "Neil Stratford <neils@vipadia.com>"
+      "Dejan Sakelsak <dejan.sakelsak@marand.si>");
 }
 
 static void
@@ -456,8 +423,6 @@ gst_rtp_h263_pay_class_init (GstRtpH263PayClass * klass)
   gobject_class = (GObjectClass *) klass;
   gstbasertppayload_class = (GstBaseRTPPayloadClass *) klass;
 
-  parent_class = g_type_class_peek_parent (klass);
-
   gobject_class->finalize = gst_rtp_h263_pay_finalize;
 
   gstbasertppayload_class->set_caps = gst_rtp_h263_pay_setcaps;
@@ -476,7 +441,7 @@ gst_rtp_h263_pay_class_init (GstRtpH263PayClass * klass)
 }
 
 static void
-gst_rtp_h263_pay_init (GstRtpH263Pay * rtph263pay)
+gst_rtp_h263_pay_init (GstRtpH263Pay * rtph263pay, GstRtpH263PayClass * klass)
 {
   rtph263pay->adapter = gst_adapter_new ();
 
@@ -566,7 +531,7 @@ gst_rtp_h263_pay_get_property (GObject * object, guint prop_id,
 }
 
 static GstRtpH263PayPackage *
-gst_rtp_h263_pay_package_new_empty ()
+gst_rtp_h263_pay_package_new_empty (void)
 {
   return (GstRtpH263PayPackage *) g_malloc0 (sizeof (GstRtpH263PayPackage));
 }
@@ -598,27 +563,14 @@ gst_rtp_h263_pay_package_destroy (GstRtpH263PayPackage * pack)
     g_free (pack);
 }
 
-static GstRtpH263PayBoundry *
-gst_rtp_h263_pay_boundry_new (guint8 * start, guint8 * end, guint8 sbit,
-    guint8 ebit)
+static void
+gst_rtp_h263_pay_boundry_init (GstRtpH263PayBoundry * boundry,
+    guint8 * start, guint8 * end, guint8 sbit, guint8 ebit)
 {
-  GstRtpH263PayBoundry *boundry;
-
-  boundry = (GstRtpH263PayBoundry *) g_malloc0 (sizeof (GstRtpH263PayBoundry));
-
   boundry->start = start;
   boundry->end = end;
   boundry->sbit = sbit;
   boundry->ebit = ebit;
-
-  return boundry;
-}
-
-static void
-gst_rtp_h263_pay_boundry_destroy (GstRtpH263PayBoundry * boundry)
-{
-  if (boundry)
-    g_free (boundry);
 }
 
 static void
@@ -698,14 +650,13 @@ gst_rtp_h263_pay_splat_header_B (guint8 * header,
 
 }
 
-static GstRtpH263PayBoundry *
+static gboolean
 gst_rtp_h263_pay_gobfinder (GstRtpH263Pay * rtph263pay,
     GstRtpH263PayBoundry * boundry)
 {
   guint8 *current;
   guint range;
   guint i;
-  GstRtpH263PayBoundry *b;
 
   current = boundry->end + 1;
   range = (rtph263pay->data - current) + rtph263pay->available_data;
@@ -717,28 +668,26 @@ gst_rtp_h263_pay_gobfinder (GstRtpH263Pay * rtph263pay,
 
   /* If we are past the end, stop */
   if (current >= rtph263pay->data + rtph263pay->available_data)
-    return NULL;
+    return FALSE;
 
   for (i = 3; i < range - 3; i++) {
     if ((current[i] == 0x0) &&
         (current[i + 1] == 0x0) && (current[i + 2] >> 7 == 0x1)) {
-      GST_LOG ("GOB end found at: %p start: %p len: %d", current + i - 1,
-          boundry->end + 1, current + i - boundry->end + 2);
-      b = gst_rtp_h263_pay_boundry_new (boundry->end + 1, current + i - 1, 0,
-          0);
-      gst_rtp_h263_pay_boundry_destroy (boundry);
+      GST_LOG ("GOB end found at: %p start: %p len: %u", current + i - 1,
+          boundry->end + 1, (guint) (current + i - boundry->end + 2));
+      gst_rtp_h263_pay_boundry_init (boundry, boundry->end + 1,
+          current + i - 1, 0, 0);
 
-      return b;
+      return TRUE;
     }
   }
 
   GST_DEBUG ("Couldn't find any new GBSC in this frame, range:%u", range);
 
-  b = gst_rtp_h263_pay_boundry_new (boundry->end + 1,
+  gst_rtp_h263_pay_boundry_init (boundry, boundry->end + 1,
       (guint8 *) (rtph263pay->data + rtph263pay->available_data - 1), 0, 0);
-  gst_rtp_h263_pay_boundry_destroy (boundry);
 
-  return b;
+  return TRUE;
 }
 
 static GstRtpH263PayGob *
@@ -941,8 +890,9 @@ gst_rtp_h263_pay_move_window_right (GstRtpH263PayContext * context, guint n,
     } else {
       if (n > rest_bits) {
         context->window =
-            (context->window << rest_bits) | (*context->
-            win_end & (((guint) pow (2.0, (double) rest_bits)) - 1));
+            (context->
+            window << rest_bits) | (*context->win_end & (((guint) pow (2.0,
+                        (double) rest_bits)) - 1));
         n -= rest_bits;
         rest_bits = 0;
       } else {
@@ -981,10 +931,13 @@ gst_rtp_h263_pay_B_mbfinder (GstRtpH263PayContext * context,
   guint cbpy_type_index;
   guint tcoef_type_index;
   GstRtpH263PayMB *mac;
+  GstRtpH263PayBoundry boundry;
+
+
+  gst_rtp_h263_pay_boundry_init (&boundry, macroblock->end,
+      macroblock->end, 8 - macroblock->ebit, macroblock->ebit);
+  mac = gst_rtp_h263_pay_mb_new (&boundry, mba);
 
-  mac =
-      gst_rtp_h263_pay_mb_new (gst_rtp_h263_pay_boundry_new (macroblock->end,
-          macroblock->end, 8 - macroblock->ebit, macroblock->ebit), mba);
 
   if (mac->sbit == 8) {
     mac->start++;
@@ -1311,7 +1264,7 @@ gst_rtp_h263_pay_push (GstRtpH263Pay * rtph263pay,
   /*
    * Copy the payload data in the buffer
    */
-  GST_DEBUG ("Copiing memory");
+  GST_DEBUG ("Copying memory");
   memcpy (payload, (guint8 *) package->payload_start, package->payload_len);
 
   /*
@@ -1349,7 +1302,7 @@ gst_rtp_h263_pay_A_fragment_push (GstRtpH263Pay * rtph263pay,
       (context->gobs[last]->end - context->gobs[first]->start) + 1;
   pack->marker = FALSE;
 
-  if (last == format_props[context->piclayer->ptype_srcformat][0] - 1) {
+  if (last == context->no_gobs - 1) {
     pack->marker = TRUE;
   }
 
@@ -1428,7 +1381,7 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
     /*---------- MODE B MODE FRAGMENTATION ----------*/
   GstRtpH263PayMB *mac;
   guint max_payload_size;
-  GstRtpH263PayBoundry *boundry;
+  GstRtpH263PayBoundry boundry;
   guint mb;
   //TODO remove m
   GstRtpH263PayMB *m;
@@ -1439,7 +1392,8 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
   max_payload_size =
       context->mtu - GST_RTP_H263_PAYLOAD_HEADER_MODE_B - GST_RTP_HEADER_LEN;
 
-  boundry = gst_rtp_h263_pay_boundry_new (gob->start, gob->start, gob->sbit, 0);
+  gst_rtp_h263_pay_boundry_init (&boundry, gob->start, gob->start, gob->sbit,
+      0);
 
   gob->macroblocks =
       (GstRtpH263PayMB **) g_malloc0 (sizeof (GstRtpH263PayMB *) *
@@ -1448,9 +1402,9 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
   GST_LOG ("GOB isn't PB frame, applying mode B");
 
   //initializing window
-  context->win_end = boundry->end;
-  if (gst_rtp_h263_pay_move_window_right (context, 32, boundry->ebit,
-          &boundry->end, &gob->end) != 0) {
+  context->win_end = boundry.end;
+  if (gst_rtp_h263_pay_move_window_right (context, 32, boundry.ebit,
+          &boundry.end, &gob->end) != 0) {
     GST_ERROR
         ("The rest of the bits should be 0, exiting, because something bad happend");
     gst_adapter_flush (rtph263pay->adapter, rtph263pay->available_data);
@@ -1462,9 +1416,9 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
     GST_LOG ("Initial GOB");
     shift = 43;
 
-    boundry->ebit =
-        gst_rtp_h263_pay_move_window_right (context, shift, boundry->ebit,
-        &boundry->end, &gob->end);
+    boundry.ebit =
+        gst_rtp_h263_pay_move_window_right (context, shift, boundry.ebit,
+        &boundry.end, &gob->end);
 
     //We need PQUANT for mode B packages - so we store it
     gob->quant = context->window >> 27;
@@ -1474,24 +1428,24 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
     shift = 6;
     if (context->cpm == 1)
       shift += 2;
-    boundry->ebit =
-        gst_rtp_h263_pay_move_window_right (context, shift, boundry->ebit,
-        &boundry->end, &gob->end);
+    boundry.ebit =
+        gst_rtp_h263_pay_move_window_right (context, shift, boundry.ebit,
+        &boundry.end, &gob->end);
 
     GST_DEBUG ("window: 0x%08x", context->window);
 
     //Shifting the PEI and PSPARE fields
     while ((context->window & 0x80000000) == 0x80000000) {
-      boundry->ebit =
+      boundry.ebit =
           gst_rtp_h263_pay_move_window_right (context, 9,
-          boundry->ebit, &boundry->end, &gob->end);
+          boundry.ebit, &boundry.end, &gob->end);
       GST_LOG ("window: 0x%x", context->window);
     }
 
     //shift the last PEI field
-    boundry->ebit =
-        gst_rtp_h263_pay_move_window_right (context, 1, boundry->ebit,
-        &boundry->end, &gob->end);
+    boundry.ebit =
+        gst_rtp_h263_pay_move_window_right (context, 1, boundry.ebit,
+        &boundry.end, &gob->end);
 
   } else {
     //skipping GOBs 24 header bits + 5 GQUANT
@@ -1504,17 +1458,17 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
       shift += 2;
 
     GST_LOG ("window: 0x%x", context->window);
-    boundry->ebit =
+    boundry.ebit =
         gst_rtp_h263_pay_move_window_right (context, shift,
-        boundry->ebit, &boundry->end, &gob->end);
+        boundry.ebit, &boundry.end, &gob->end);
 
     //We need GQUANT for mode B packages - so we store it
     gob->quant = context->window >> 27;
 
     shift = 5;
-    boundry->ebit =
+    boundry.ebit =
         gst_rtp_h263_pay_move_window_right (context, shift,
-        boundry->ebit, &boundry->end, &gob->end);
+        boundry.ebit, &boundry.end, &gob->end);
 
     GST_LOG ("window: 0x%x", context->window);
   }
@@ -1523,7 +1477,7 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
 
   // We are on MB layer
 
-  m = mac = gst_rtp_h263_pay_mb_new (boundry, 0);
+  m = mac = gst_rtp_h263_pay_mb_new (&boundry, 0);
   for (mb = 0; mb < format_props[context->piclayer->ptype_srcformat][1]; mb++) {
 
     GST_DEBUG ("================ START MB %d =================", mb);
@@ -1562,7 +1516,7 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
 
   mb = 0;
   first = 0;
-  payload_len = boundry->end - boundry->start + 1;
+  payload_len = boundry.end - boundry.start + 1;
   GST_DEBUG ("------------------------- NEW PACKAGE ----------------------");
   while (mb < gob->nmacroblocs) {
     if (payload_len + gob->macroblocks[mb]->length < max_payload_size) {
@@ -1577,7 +1531,7 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
       //    first == mb - 1, payload_len);
 
       if (gst_rtp_h263_pay_B_fragment_push (rtph263pay, context, gob, first,
-              mb - 1, boundry)) {
+              mb - 1, &boundry)) {
         GST_ERROR ("Oooops, there was an error sending");
         return FALSE;
       }
@@ -1595,7 +1549,7 @@ gst_rtp_h263_pay_mode_B_fragment (GstRtpH263Pay * rtph263pay,
   GST_DEBUG ("Remainder first: %d, MB: %d", first, mb);
   if (payload_len != 0) {
     if (gst_rtp_h263_pay_B_fragment_push (rtph263pay, context, gob, first,
-            mb - 1, boundry)) {
+            mb - 1, &boundry)) {
       GST_ERROR ("Oooops, there was an error sending!");
       return FALSE;
     }
@@ -1688,40 +1642,48 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
   } else {
 
       /*---------- FRAGMENTING THE FRAME BECAUSE TOO LARGE TO FIT IN MTU ----------*/
-    GstRtpH263PayBoundry *bound;
+    GstRtpH263PayBoundry bound;
     gint first;
     guint payload_len;
+    gboolean forcea = FALSE;
 
     GST_DEBUG ("Frame too large for MTU");
     /*
      * Let's go trough all the data and fragment it untill end is reached
      */
 
-    bound = gst_rtp_h263_pay_boundry_new (NULL, rtph263pay->data - 1, 0, 0);
+    gst_rtp_h263_pay_boundry_init (&bound, NULL, rtph263pay->data - 1, 0, 0);
     context->gobs =
-        (GstRtpH263PayGob **) g_malloc0 (format_props[context->piclayer->
-            ptype_srcformat][0] * sizeof (GstRtpH263PayGob *));
+        (GstRtpH263PayGob **) g_malloc0 (format_props[context->
+            piclayer->ptype_srcformat][0] * sizeof (GstRtpH263PayGob *));
 
 
     for (i = 0; i < format_props[context->piclayer->ptype_srcformat][0]; i++) {
       GST_DEBUG ("Searching for gob %d", i);
-      bound = gst_rtp_h263_pay_gobfinder (rtph263pay, bound);
-      if (!bound) {
-        GST_WARNING
-            ("No GOB's were found in data stream! Please enable RTP mode in encoder. Forcing mode A for now.");
-        ret = gst_rtp_h263_send_entire_frame (rtph263pay, context);
-        goto end;
+      if (!gst_rtp_h263_pay_gobfinder (rtph263pay, &bound)) {
+        if (i <= 1) {
+          GST_WARNING
+              ("No GOB's were found in data stream! Please enable RTP mode in encoder. Forcing mode A for now.");
+          ret = gst_rtp_h263_send_entire_frame (rtph263pay, context);
+          goto end;
+        } else {
+          /* try to send fragments corresponding to found GOBs */
+          forcea = TRUE;
+          break;
+        }
       }
 
-      context->gobs[i] = gst_rtp_h263_pay_gob_new (bound, i);
+      context->gobs[i] = gst_rtp_h263_pay_gob_new (&bound, i);
       //FIXME - encoders may generate an EOS gob that has to be processed
       GST_DEBUG
           ("Gob values are: gobn: %d, start: %p len: %d ebit: %d sbit: %d", i,
           context->gobs[i]->start, context->gobs[i]->length,
           context->gobs[i]->ebit, context->gobs[i]->sbit);
     }
-
-    gst_rtp_h263_pay_boundry_destroy (bound);
+    /* NOTE some places may still assume this to be the max possible */
+    context->no_gobs = i;
+    GST_DEBUG ("Found %d GOBS of maximum %d",
+        context->no_gobs, format_props[context->piclayer->ptype_srcformat][0]);
 
     // Make packages smaller than MTU
     //   A mode
@@ -1731,13 +1693,13 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
     first = 0;
     payload_len = 0;
     i = 0;
-    while (i < format_props[context->piclayer->ptype_srcformat][0]) {
+    while (i < context->no_gobs) {
 
       if (context->gobs[i]->length >= context->mtu) {
         if (payload_len == 0) {
 
           GST_DEBUG ("GOB len > MTU");
-          if (rtph263pay->prop_payload_mode) {
+          if (rtph263pay->prop_payload_mode || forcea) {
             payload_len = context->gobs[i]->length;
             goto force_a;
           }
@@ -1772,7 +1734,7 @@ gst_rtp_h263_pay_flush (GstRtpH263Pay * rtph263pay)
         GST_DEBUG ("GOB %d fills mtu", i);
         payload_len += context->gobs[i]->length;
         i++;
-        if (i == format_props[context->piclayer->ptype_srcformat][0]) {
+        if (i == context->no_gobs) {
           GST_DEBUG ("LAST GOB %d", i);
           goto payload_a_push;
         }
