@@ -67,8 +67,6 @@
 #include "rtpjitterbuffer.h"
 #include "rtpstats.h"
 
-#include <gst/glib-compat-private.h>
-
 GST_DEBUG_CATEGORY (rtpjitterbuffer_debug);
 #define GST_CAT_DEFAULT (rtpjitterbuffer_debug)
 
@@ -237,7 +235,6 @@ static GstPad *gst_rtp_jitter_buffer_request_new_pad (GstElement * element,
     GstPadTemplate * templ, const gchar * name);
 static void gst_rtp_jitter_buffer_release_pad (GstElement * element,
     GstPad * pad);
-static GstClock *gst_rtp_jitter_buffer_provide_clock (GstElement * element);
 
 /* pad overrides */
 static GstCaps *gst_rtp_jitter_buffer_getcaps (GstPad * pad);
@@ -274,12 +271,12 @@ gst_rtp_jitter_buffer_base_init (gpointer klass)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (klass);
 
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jitter_buffer_src_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jitter_buffer_sink_template);
-  gst_element_class_add_static_pad_template (element_class,
-      &gst_rtp_jitter_buffer_sink_rtcp_template);
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jitter_buffer_src_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jitter_buffer_sink_template));
+  gst_element_class_add_pad_template (element_class,
+      gst_static_pad_template_get (&gst_rtp_jitter_buffer_sink_rtcp_template));
 
   gst_element_class_set_details_simple (element_class,
       "RTP packet jitter-buffer", "Filter/Network/RTP",
@@ -299,7 +296,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
 
   g_type_class_add_private (klass, sizeof (GstRtpJitterBufferPrivate));
 
-  gobject_class->finalize = gst_rtp_jitter_buffer_finalize;
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_finalize);
 
   gobject_class->set_property = gst_rtp_jitter_buffer_set_property;
   gobject_class->get_property = gst_rtp_jitter_buffer_get_property;
@@ -313,7 +310,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
   g_object_class_install_property (gobject_class, PROP_LATENCY,
       g_param_spec_uint ("latency", "Buffer latency in ms",
           "Amount of ms to buffer", 0, G_MAXUINT, DEFAULT_LATENCY_MS,
-          G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READWRITE));
   /**
    * GstRtpJitterBuffer::drop-on-latency:
    *
@@ -323,7 +320,7 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
       g_param_spec_boolean ("drop-on-latency",
           "Drop buffers when maximum latency is reached",
           "Tells the jitterbuffer to never exceed the given latency in size",
-          DEFAULT_DROP_ON_LATENCY, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_DROP_ON_LATENCY, G_PARAM_READWRITE));
   /**
    * GstRtpJitterBuffer::ts-offset:
    *
@@ -442,8 +439,6 @@ gst_rtp_jitter_buffer_class_init (GstRtpJitterBufferClass * klass)
       GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_request_new_pad);
   gstelement_class->release_pad =
       GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_release_pad);
-  gstelement_class->provide_clock =
-      GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_provide_clock);
 
   klass->clear_pt_map = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_clear_pt_map);
   klass->set_active = GST_DEBUG_FUNCPTR (gst_rtp_jitter_buffer_set_active);
@@ -656,12 +651,6 @@ wrong_pad:
   }
 }
 
-static GstClock *
-gst_rtp_jitter_buffer_provide_clock (GstElement * element)
-{
-  return gst_system_clock_obtain ();
-}
-
 static void
 gst_rtp_jitter_buffer_clear_pt_map (GstRtpJitterBuffer * jitterbuffer)
 {
@@ -673,11 +662,6 @@ gst_rtp_jitter_buffer_clear_pt_map (GstRtpJitterBuffer * jitterbuffer)
 
   JBUF_LOCK (priv);
   priv->clock_rate = -1;
-  /* do not clear current content, but refresh state for new arrival */
-  GST_DEBUG_OBJECT (jitterbuffer, "reset jitterbuffer");
-  rtp_jitter_buffer_reset_skew (priv->jbuf);
-  priv->last_popped_seqnum = -1;
-  priv->next_seqnum = -1;
   JBUF_UNLOCK (priv);
 }
 
@@ -1015,10 +999,6 @@ gst_rtp_jitter_buffer_src_event (GstPad * pad, GstEvent * event)
   GstRtpJitterBufferPrivate *priv;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (jitterbuffer == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
   priv = jitterbuffer->priv;
 
   GST_DEBUG_OBJECT (jitterbuffer, "received %s", GST_EVENT_TYPE_NAME (event));
@@ -1060,10 +1040,6 @@ gst_rtp_jitter_buffer_sink_event (GstPad * pad, GstEvent * event)
   GstRtpJitterBufferPrivate *priv;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (jitterbuffer == NULL)) {
-    gst_event_unref (event);
-    return FALSE;
-  }
   priv = jitterbuffer->priv;
 
   GST_DEBUG_OBJECT (jitterbuffer, "received %s", GST_EVENT_TYPE_NAME (event));
@@ -1114,7 +1090,7 @@ gst_rtp_jitter_buffer_sink_event (GstPad * pad, GstEvent * event)
        * we are flushing */
       ret = priv->srcresult == GST_FLOW_OK;
       if (ret && !priv->eos) {
-        GST_INFO_OBJECT (jitterbuffer, "queuing EOS");
+        GST_DEBUG_OBJECT (jitterbuffer, "queuing EOS");
         priv->eos = TRUE;
         JBUF_SIGNAL (priv);
       } else if (priv->eos) {
@@ -1142,7 +1118,6 @@ newseg_wrong_format:
   {
     GST_DEBUG_OBJECT (jitterbuffer, "received non TIME newsegment");
     ret = FALSE;
-    gst_event_unref (event);
     goto done;
   }
 }
@@ -1151,8 +1126,10 @@ static gboolean
 gst_rtp_jitter_buffer_sink_rtcp_event (GstPad * pad, GstEvent * event)
 {
   GstRtpJitterBuffer *jitterbuffer;
+  GstRtpJitterBufferPrivate *priv;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
+  priv = jitterbuffer->priv;
 
   GST_DEBUG_OBJECT (jitterbuffer, "received %s", GST_EVENT_TYPE_NAME (event));
 
@@ -1231,22 +1208,6 @@ parse_failed:
   }
 }
 
-/* call with jbuf lock held */
-static void
-check_buffering_percent (GstRtpJitterBuffer * jitterbuffer, gint * percent)
-{
-  GstRtpJitterBufferPrivate *priv = jitterbuffer->priv;
-
-  /* too short a stream, or too close to EOS will never really fill buffer */
-  if (*percent != -1 && priv->npt_stop != -1 &&
-      priv->npt_stop - priv->npt_start <=
-      rtp_jitter_buffer_get_delay (priv->jbuf)) {
-    GST_DEBUG_OBJECT (jitterbuffer, "short stream; faking full buffer");
-    rtp_jitter_buffer_set_buffering (priv->jbuf, FALSE);
-    *percent = 100;
-  }
-}
-
 static void
 post_buffering_percent (GstRtpJitterBuffer * jitterbuffer, gint percent)
 {
@@ -1421,8 +1382,6 @@ gst_rtp_jitter_buffer_chain (GstPad * pad, GstBuffer * buffer)
   GST_DEBUG_OBJECT (jitterbuffer, "Pushed packet #%d, now %d packets, tail: %d",
       seqnum, rtp_jitter_buffer_num_packets (priv->jbuf), tail);
 
-  check_buffering_percent (jitterbuffer, &percent);
-
 finished:
   JBUF_UNLOCK (priv);
 
@@ -1526,7 +1485,7 @@ eos_reached (GstClock * clock, GstClockTime time, GstClockID id,
 
   JBUF_LOCK_CHECK (priv, flushing);
   if (priv->waiting) {
-    GST_INFO_OBJECT (jitterbuffer, "got the NPT timeout");
+    GST_DEBUG_OBJECT (jitterbuffer, "got the NPT timeout");
     priv->reached_npt_stop = TRUE;
     JBUF_SIGNAL (priv);
   }
@@ -1542,34 +1501,6 @@ flushing:
   }
 }
 
-static GstClockTime
-compute_elapsed (GstRtpJitterBuffer * jitterbuffer, GstBuffer * outbuf)
-{
-  guint64 ext_time, elapsed;
-  guint32 rtp_time;
-  GstRtpJitterBufferPrivate *priv;
-
-  priv = jitterbuffer->priv;
-  rtp_time = gst_rtp_buffer_get_timestamp (outbuf);
-
-  GST_LOG_OBJECT (jitterbuffer, "rtp %" G_GUINT32_FORMAT ", ext %"
-      G_GUINT64_FORMAT, rtp_time, priv->ext_timestamp);
-
-  if (rtp_time < priv->ext_timestamp) {
-    ext_time = priv->ext_timestamp;
-  } else {
-    ext_time = gst_rtp_buffer_ext_timestamp (&priv->ext_timestamp, rtp_time);
-  }
-
-  if (ext_time > priv->clock_base)
-    elapsed = ext_time - priv->clock_base;
-  else
-    elapsed = 0;
-
-  elapsed = gst_util_uint64_scale_int (elapsed, GST_SECOND, priv->clock_rate);
-  return elapsed;
-}
-
 /*
  * This funcion will push out buffers on the source pad.
  *
@@ -1604,37 +1535,8 @@ again:
     if (G_LIKELY (!priv->blocked)) {
       /* we're buffering but not EOS, wait. */
       if (!priv->eos && (!priv->active
-              || rtp_jitter_buffer_is_buffering (priv->jbuf))) {
-        GstClockTime elapsed, delay, left;
-
-        if (priv->estimated_eos == -1)
-          goto do_wait;
-
-        outbuf = rtp_jitter_buffer_peek (priv->jbuf);
-        if (outbuf != NULL) {
-          elapsed = compute_elapsed (jitterbuffer, outbuf);
-          if (GST_BUFFER_DURATION_IS_VALID (outbuf))
-            elapsed += GST_BUFFER_DURATION (outbuf);
-        } else {
-          GST_INFO_OBJECT (jitterbuffer, "no buffer, using last_elapsed");
-          elapsed = priv->last_elapsed;
-        }
-
-        delay = rtp_jitter_buffer_get_delay (priv->jbuf);
-
-        if (priv->estimated_eos > elapsed)
-          left = priv->estimated_eos - elapsed;
-        else
-          left = 0;
-
-        GST_INFO_OBJECT (jitterbuffer, "buffering, elapsed %" GST_TIME_FORMAT
-            " estimated_eos %" GST_TIME_FORMAT " left %" GST_TIME_FORMAT
-            " delay %" GST_TIME_FORMAT,
-            GST_TIME_ARGS (elapsed), GST_TIME_ARGS (priv->estimated_eos),
-            GST_TIME_ARGS (left), GST_TIME_ARGS (delay));
-        if (left > delay)
-          goto do_wait;
-      }
+              || rtp_jitter_buffer_is_buffering (priv->jbuf)))
+        goto do_wait;
       /* if we have a packet, we can exit the loop and grab it */
       if (rtp_jitter_buffer_num_packets (priv->jbuf) > 0)
         break;
@@ -1649,7 +1551,7 @@ again:
         GST_OBJECT_LOCK (jitterbuffer);
         clock = GST_ELEMENT_CLOCK (jitterbuffer);
         if (clock) {
-          GST_INFO_OBJECT (jitterbuffer, "scheduling timeout");
+          GST_DEBUG_OBJECT (jitterbuffer, "scheduling timeout");
           id = gst_clock_new_single_shot_id (clock, sync_time);
           gst_clock_id_wait_async (id, (GstClockCallback) eos_reached,
               jitterbuffer);
@@ -1817,7 +1719,7 @@ again:
       GstEvent *event;
 
       /* we had a gap and thus we lost a packet. Create an event for this.  */
-      GST_DEBUG_OBJECT (jitterbuffer, "Packet #%d lost", next_seqnum);
+      GST_WARNING_OBJECT (jitterbuffer, "Packet #%d lost", next_seqnum);
       priv->num_late++;
       discont = TRUE;
 
@@ -1853,8 +1755,6 @@ push_buffer:
   /* when we get here we are ready to pop and push the buffer */
   outbuf = rtp_jitter_buffer_pop (priv->jbuf, &percent);
 
-  check_buffering_percent (jitterbuffer, &percent);
-
   if (G_UNLIKELY (discont || priv->discont)) {
     /* set DISCONT flag when we missed a packet. We pushed the buffer writable
      * into the jitterbuffer so we can modify now. */
@@ -1868,29 +1768,38 @@ push_buffer:
   /* update the elapsed time when we need to check against the npt stop time. */
   if (priv->npt_stop != -1 && priv->ext_timestamp != -1
       && priv->clock_base != -1 && priv->clock_rate > 0) {
-    guint64 elapsed, estimated;
+    guint64 ext_time, elapsed, estimated;
+    guint32 rtp_time;
 
-    elapsed = compute_elapsed (jitterbuffer, outbuf);
+    rtp_time = gst_rtp_buffer_get_timestamp (outbuf);
 
-    if (elapsed > priv->last_elapsed || !priv->last_elapsed) {
+    GST_LOG_OBJECT (jitterbuffer, "rtp %" G_GUINT32_FORMAT ", ext %"
+        G_GUINT64_FORMAT, rtp_time, priv->ext_timestamp);
+
+    if (rtp_time < priv->ext_timestamp) {
+      ext_time = priv->ext_timestamp;
+    } else {
+      ext_time = gst_rtp_buffer_ext_timestamp (&priv->ext_timestamp, rtp_time);
+    }
+
+    if (ext_time > priv->clock_base)
+      elapsed = ext_time - priv->clock_base;
+    else
+      elapsed = 0;
+
+    elapsed = gst_util_uint64_scale_int (elapsed, GST_SECOND, priv->clock_rate);
+
+    if (elapsed > priv->last_elapsed) {
       guint64 left;
 
       priv->last_elapsed = elapsed;
 
       left = priv->npt_stop - priv->npt_start;
-      GST_LOG_OBJECT (jitterbuffer, "left %" GST_TIME_FORMAT,
-          GST_TIME_ARGS (left));
 
       if (elapsed > 0)
         estimated = gst_util_uint64_scale (out_time, left, elapsed);
-      else {
-        /* if there is almost nothing left,
-         * we may never advance enough to end up in the above case */
-        if (left < GST_SECOND)
-          estimated = GST_SECOND;
-        else
-          estimated = -1;
-      }
+      else
+        estimated = -1;
 
       GST_LOG_OBJECT (jitterbuffer, "elapsed %" GST_TIME_FORMAT ", estimated %"
           GST_TIME_FORMAT, GST_TIME_ARGS (elapsed), GST_TIME_ARGS (estimated));
@@ -1969,7 +1878,7 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   GstRtpJitterBuffer *jitterbuffer;
   GstRtpJitterBufferPrivate *priv;
   GstFlowReturn ret = GST_FLOW_OK;
-  guint64 base_rtptime, base_time;
+  guint64 base_rtptime, timestamp;
   guint32 clock_rate;
   guint64 last_rtptime;
   guint32 ssrc;
@@ -1977,7 +1886,6 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   guint64 ext_rtptime, diff;
   guint32 rtptime;
   gboolean drop = FALSE;
-  guint64 clock_base;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
 
@@ -2008,17 +1916,14 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
   ext_rtptime = gst_rtp_buffer_ext_timestamp (&ext_rtptime, rtptime);
 
   /* get the last values from the jitterbuffer */
-  rtp_jitter_buffer_get_sync (priv->jbuf, &base_rtptime, &base_time,
+  rtp_jitter_buffer_get_sync (priv->jbuf, &base_rtptime, &timestamp,
       &clock_rate, &last_rtptime);
 
-  clock_base = priv->clock_base;
-
   GST_DEBUG_OBJECT (jitterbuffer, "ext SR %" G_GUINT64_FORMAT ", base %"
-      G_GUINT64_FORMAT ", clock-rate %" G_GUINT32_FORMAT
-      ", clock-base %" G_GUINT64_FORMAT,
-      ext_rtptime, base_rtptime, clock_rate, clock_base);
+      G_GUINT64_FORMAT ", clock-rate %" G_GUINT32_FORMAT,
+      ext_rtptime, base_rtptime, clock_rate);
 
-  if (base_rtptime == -1 || clock_rate == -1 || base_time == -1) {
+  if (base_rtptime == -1 || clock_rate == -1 || timestamp == -1) {
     GST_DEBUG_OBJECT (jitterbuffer, "dropping, no RTP values");
     drop = TRUE;
   } else {
@@ -2034,12 +1939,8 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
         diff = ext_rtptime - last_rtptime;
         /* if bigger than 1 second, we drop it */
         if (diff > clock_rate) {
-          GST_DEBUG_OBJECT (jitterbuffer, "too far ahead");
-          /* should drop this, but some RTSP servers end up with bogus
-           * way too ahead RTCP packet when repeated PAUSE/PLAY,
-           * so still trigger rptbin sync but invalidate RTCP data
-           * (sync might use other methods) */
-          ext_rtptime = -1;
+          GST_DEBUG_OBJECT (jitterbuffer, "dropping, too far ahead");
+          drop = TRUE;
         }
         GST_DEBUG_OBJECT (jitterbuffer, "ext last %" G_GUINT64_FORMAT ", diff %"
             G_GUINT64_FORMAT, last_rtptime, diff);
@@ -2053,9 +1954,8 @@ gst_rtp_jitter_buffer_chain_rtcp (GstPad * pad, GstBuffer * buffer)
 
     s = gst_structure_new ("application/x-rtp-sync",
         "base-rtptime", G_TYPE_UINT64, base_rtptime,
-        "base-time", G_TYPE_UINT64, base_time,
+        "base-time", G_TYPE_UINT64, timestamp,
         "clock-rate", G_TYPE_UINT, clock_rate,
-        "clock-base", G_TYPE_UINT64, clock_base,
         "sr-ext-rtptime", G_TYPE_UINT64, ext_rtptime,
         "sr-buffer", GST_TYPE_BUFFER, buffer, NULL);
 
@@ -2098,8 +1998,6 @@ gst_rtp_jitter_buffer_query (GstPad * pad, GstQuery * query)
   gboolean res = FALSE;
 
   jitterbuffer = GST_RTP_JITTER_BUFFER (gst_pad_get_parent (pad));
-  if (G_UNLIKELY (jitterbuffer == NULL))
-    return FALSE;
   priv = jitterbuffer->priv;
 
   switch (GST_QUERY_TYPE (query)) {
@@ -2139,35 +2037,6 @@ gst_rtp_jitter_buffer_query (GstPad * pad, GstQuery * query)
       }
       break;
     }
-    case GST_QUERY_POSITION:
-    {
-      GstClockTime start, last_out;
-      GstFormat fmt;
-
-      gst_query_parse_position (query, &fmt, NULL);
-      if (fmt != GST_FORMAT_TIME) {
-        res = gst_pad_query_default (pad, query);
-        break;
-      }
-
-      JBUF_LOCK (priv);
-      start = priv->npt_start;
-      last_out = priv->last_out_time;
-      JBUF_UNLOCK (priv);
-
-      GST_DEBUG_OBJECT (jitterbuffer, "npt start %" GST_TIME_FORMAT
-          ", last out %" GST_TIME_FORMAT, GST_TIME_ARGS (start),
-          GST_TIME_ARGS (last_out));
-
-      if (GST_CLOCK_TIME_IS_VALID (start) && GST_CLOCK_TIME_IS_VALID (last_out)) {
-        /* bring 0-based outgoing time to stream time */
-        gst_query_set_position (query, GST_FORMAT_TIME, start + last_out);
-        res = TRUE;
-      } else {
-        res = gst_pad_query_default (pad, query);
-      }
-      break;
-    }
     default:
       res = gst_pad_query_default (pad, query);
       break;
