@@ -63,13 +63,6 @@
 GST_DEBUG_CATEGORY_STATIC (rtpdec_debug);
 #define GST_CAT_DEFAULT (rtpdec_debug)
 
-/* elementfactory information */
-static const GstElementDetails rtpdec_details =
-GST_ELEMENT_DETAILS ("RTP Decoder",
-    "Codec/Parser/Network",
-    "Accepts raw RTP and RTCP packets and sends them forward",
-    "Wim Taymans <wim@fluendo.com>");
-
 /* GstRTPDec signals and args */
 enum
 {
@@ -218,13 +211,16 @@ gst_rtp_dec_base_init (gpointer klass)
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&gst_rtp_dec_rtcp_src_template));
 
-  gst_element_class_set_details (element_class, &rtpdec_details);
+  gst_element_class_set_details_simple (element_class, "RTP Decoder",
+      "Codec/Parser/Network",
+      "Accepts raw RTP and RTCP packets and sends them forward",
+      "Wim Taymans <wim@fluendo.com>");
 }
 
 /* BOXED:UINT,UINT */
 #define g_marshal_value_peek_uint(v)     g_value_get_uint (v)
 
-void
+static void
 gst_rtp_dec_marshal_BOXED__UINT_UINT (GClosure * closure,
     GValue * return_value,
     guint n_param_values,
@@ -259,7 +255,7 @@ gst_rtp_dec_marshal_BOXED__UINT_UINT (GClosure * closure,
   g_value_take_boxed (return_value, v_return);
 }
 
-void
+static void
 gst_rtp_dec_marshal_VOID__UINT_UINT (GClosure * closure,
     GValue * return_value,
     guint n_param_values,
