@@ -88,9 +88,6 @@ GST_DEBUG_CATEGORY (gst_videomixer_debug);
 
 static GType gst_videomixer_get_type (void);
 
-static void gst_videomixer_pad_class_init (GstVideoMixerPadClass * klass);
-static void gst_videomixer_pad_init (GstVideoMixerPad * mixerpad);
-
 static void gst_videomixer_pad_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 static void gst_videomixer_pad_set_property (GObject * object, guint prop_id,
@@ -114,6 +111,7 @@ enum
   PROP_PAD_ALPHA
 };
 
+GType gst_videomixer_pad_get_type (void);
 G_DEFINE_TYPE (GstVideoMixerPad, gst_videomixer_pad, GST_TYPE_PAD);
 
 static void
@@ -472,20 +470,28 @@ static GstStaticPadTemplate src_factory = GST_STATIC_PAD_TEMPLATE ("src",
     GST_PAD_SRC,
     GST_PAD_ALWAYS,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV") ";" GST_VIDEO_CAPS_BGRA ";"
-        GST_VIDEO_CAPS_ARGB ";" GST_VIDEO_CAPS_YUV ("I420")
-        ";" GST_VIDEO_CAPS_RGB ";" GST_VIDEO_CAPS_BGR
-        ";" GST_VIDEO_CAPS_xRGB ";" GST_VIDEO_CAPS_xBGR
-        ";" GST_VIDEO_CAPS_RGBx ";" GST_VIDEO_CAPS_BGRx)
+        GST_VIDEO_CAPS_ARGB ";" GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_ABGR ";"
+        GST_VIDEO_CAPS_YUV ("Y444") ";" GST_VIDEO_CAPS_YUV ("Y42B") ";"
+        GST_VIDEO_CAPS_YUV ("YUY2") ";" GST_VIDEO_CAPS_YUV ("UYVY") ";"
+        GST_VIDEO_CAPS_YUV ("YVYU") ";"
+        GST_VIDEO_CAPS_YUV ("I420") ";" GST_VIDEO_CAPS_YUV ("YV12") ";"
+        GST_VIDEO_CAPS_YUV ("Y41B") ";" GST_VIDEO_CAPS_RGB ";"
+        GST_VIDEO_CAPS_BGR ";" GST_VIDEO_CAPS_xRGB ";" GST_VIDEO_CAPS_xBGR ";"
+        GST_VIDEO_CAPS_RGBx ";" GST_VIDEO_CAPS_BGRx)
     );
 
 static GstStaticPadTemplate sink_factory = GST_STATIC_PAD_TEMPLATE ("sink_%d",
     GST_PAD_SINK,
     GST_PAD_REQUEST,
     GST_STATIC_CAPS (GST_VIDEO_CAPS_YUV ("AYUV") ";" GST_VIDEO_CAPS_BGRA ";"
-        GST_VIDEO_CAPS_ARGB ";" GST_VIDEO_CAPS_YUV ("I420")
-        ";" GST_VIDEO_CAPS_RGB ";" GST_VIDEO_CAPS_BGR
-        ";" GST_VIDEO_CAPS_xRGB ";" GST_VIDEO_CAPS_xBGR
-        ";" GST_VIDEO_CAPS_RGBx ";" GST_VIDEO_CAPS_BGRx)
+        GST_VIDEO_CAPS_ARGB ";" GST_VIDEO_CAPS_RGBA ";" GST_VIDEO_CAPS_ABGR ";"
+        GST_VIDEO_CAPS_YUV ("Y444") ";" GST_VIDEO_CAPS_YUV ("Y42B") ";"
+        GST_VIDEO_CAPS_YUV ("YUY2") ";" GST_VIDEO_CAPS_YUV ("UYVY") ";"
+        GST_VIDEO_CAPS_YUV ("YVYU") ";"
+        GST_VIDEO_CAPS_YUV ("I420") ";" GST_VIDEO_CAPS_YUV ("YV12") ";"
+        GST_VIDEO_CAPS_YUV ("Y41B") ";" GST_VIDEO_CAPS_RGB ";"
+        GST_VIDEO_CAPS_BGR ";" GST_VIDEO_CAPS_xRGB ";" GST_VIDEO_CAPS_xBGR ";"
+        GST_VIDEO_CAPS_RGBx ";" GST_VIDEO_CAPS_BGRx)
     );
 
 static void gst_videomixer_finalize (GObject * object);
@@ -965,12 +971,66 @@ gst_videomixer_setcaps (GstPad * pad, GstCaps * caps)
       mixer->fill_color = gst_video_mixer_fill_color_bgra;
       ret = TRUE;
       break;
+    case GST_VIDEO_FORMAT_ABGR:
+      mixer->blend = gst_video_mixer_blend_abgr;
+      mixer->fill_checker = gst_video_mixer_fill_checker_abgr;
+      mixer->fill_color = gst_video_mixer_fill_color_abgr;
+      ret = TRUE;
+      break;
+    case GST_VIDEO_FORMAT_RGBA:
+      mixer->blend = gst_video_mixer_blend_rgba;
+      mixer->fill_checker = gst_video_mixer_fill_checker_rgba;
+      mixer->fill_color = gst_video_mixer_fill_color_rgba;
+      ret = TRUE;
+      break;
+    case GST_VIDEO_FORMAT_Y444:
+      mixer->blend = gst_video_mixer_blend_y444;
+      mixer->fill_checker = gst_video_mixer_fill_checker_y444;
+      mixer->fill_color = gst_video_mixer_fill_color_y444;
+      ret = TRUE;
+      break;
+    case GST_VIDEO_FORMAT_Y42B:
+      mixer->blend = gst_video_mixer_blend_y42b;
+      mixer->fill_checker = gst_video_mixer_fill_checker_y42b;
+      mixer->fill_color = gst_video_mixer_fill_color_y42b;
+      ret = TRUE;
+      break;
+    case GST_VIDEO_FORMAT_YUY2:
+      mixer->blend = gst_video_mixer_blend_yuy2;
+      mixer->fill_checker = gst_video_mixer_fill_checker_yuy2;
+      mixer->fill_color = gst_video_mixer_fill_color_yuy2;
+      ret = TRUE;
+      break;
+    case GST_VIDEO_FORMAT_UYVY:
+      mixer->blend = gst_video_mixer_blend_uyvy;
+      mixer->fill_checker = gst_video_mixer_fill_checker_uyvy;
+      mixer->fill_color = gst_video_mixer_fill_color_uyvy;
+      ret = TRUE;
+      break;
+    case GST_VIDEO_FORMAT_YVYU:
+      mixer->blend = gst_video_mixer_blend_yvyu;
+      mixer->fill_checker = gst_video_mixer_fill_checker_yvyu;
+      mixer->fill_color = gst_video_mixer_fill_color_yvyu;
+      ret = TRUE;
+      break;
     case GST_VIDEO_FORMAT_I420:
       mixer->blend = gst_video_mixer_blend_i420;
       mixer->fill_checker = gst_video_mixer_fill_checker_i420;
       mixer->fill_color = gst_video_mixer_fill_color_i420;
       ret = TRUE;
       break;
+    case GST_VIDEO_FORMAT_YV12:
+      mixer->blend = gst_video_mixer_blend_yv12;
+      mixer->fill_checker = gst_video_mixer_fill_checker_yv12;
+      mixer->fill_color = gst_video_mixer_fill_color_yv12;
+      ret = TRUE;
+      break;
+    case GST_VIDEO_FORMAT_Y41B:
+      mixer->blend = gst_video_mixer_blend_y41b;
+      mixer->fill_checker = gst_video_mixer_fill_checker_y41b;
+      mixer->fill_color = gst_video_mixer_fill_color_y41b;
+      ret = TRUE;
+      break;
     case GST_VIDEO_FORMAT_RGB:
       mixer->blend = gst_video_mixer_blend_rgb;
       mixer->fill_checker = gst_video_mixer_fill_checker_rgb;
@@ -1041,7 +1101,9 @@ gst_videomixer_request_new_pad (GstElement * element,
     gchar *name = NULL;
     GstVideoMixerCollect *mixcol = NULL;
 
-    if (req_name == NULL || strlen (req_name) < 6) {
+    GST_VIDEO_MIXER_STATE_LOCK (mix);
+    if (req_name == NULL || strlen (req_name) < 6
+        || !g_str_has_prefix (req_name, "sink_")) {
       /* no name given when requesting the pad, use next available int */
       serial = mix->next_sinkpad++;
     } else {
@@ -1056,7 +1118,6 @@ gst_videomixer_request_new_pad (GstElement * element,
         templ->direction, "template", templ, NULL);
     g_free (name);
 
-    GST_VIDEO_MIXER_STATE_LOCK (mix);
     mixpad->zorder = mix->numpads;
     mixpad->xpos = DEFAULT_PAD_XPOS;
     mixpad->ypos = DEFAULT_PAD_YPOS;
