@@ -81,12 +81,6 @@ static gboolean gst_wavparse_sink_event (GstPad * pad, GstEvent * event);
 static void gst_wavparse_loop (GstPad * pad);
 static gboolean gst_wavparse_srcpad_event (GstPad * pad, GstEvent * event);
 
-static const GstElementDetails gst_wavparse_details =
-GST_ELEMENT_DETAILS ("WAV audio demuxer",
-    "Codec/Demuxer/Audio",
-    "Parse a .wav file into raw audio",
-    "Erik Walthinsen <omega@cse.ogi.edu>");
-
 static GstStaticPadTemplate sink_template_factory =
 GST_STATIC_PAD_TEMPLATE ("wavparse_sink",
     GST_PAD_SINK,
@@ -115,7 +109,10 @@ gst_wavparse_base_init (gpointer g_class)
   gst_element_class_add_pad_template (element_class, src_template);
   gst_object_unref (src_template);
 
-  gst_element_class_set_details (element_class, &gst_wavparse_details);
+  gst_element_class_set_details_simple (element_class, "WAV audio demuxer",
+      "Codec/Demuxer/Audio",
+      "Parse a .wav file into raw audio",
+      "Erik Walthinsen <omega@cse.ogi.edu>");
 }
 
 static void
@@ -1200,8 +1197,9 @@ gst_wavparse_stream_headers (GstWavParse * wav)
         return res;
     }
 
-    if (tag == GST_RIFF_TAG_JUNK || tag == GST_RIFF_TAG_bext ||
-        tag == GST_RIFF_TAG_BEXT || tag == GST_RIFF_TAG_LIST) {
+    if (tag == GST_RIFF_TAG_JUNK || tag == GST_RIFF_TAG_JUNQ ||
+        tag == GST_RIFF_TAG_bext || tag == GST_RIFF_TAG_BEXT ||
+        tag == GST_RIFF_TAG_LIST) {
       GST_DEBUG_OBJECT (wav, "skipping %" GST_FOURCC_FORMAT " chunk",
           GST_FOURCC_ARGS (tag));
       gst_buffer_unref (buf);
@@ -1846,6 +1844,18 @@ iterate_adapter:
     if ((res = gst_pad_pull_range (wav->sinkpad, wav->offset,
                 desired, &buf)) != GST_FLOW_OK)
       goto pull_error;
+
+    /* we may get a short buffer at the end of the file */
+    if (GST_BUFFER_SIZE (buf) < desired) {
+      GST_LOG_OBJECT (wav, "Got only %u bytes of data", GST_BUFFER_SIZE (buf));
+      if (GST_BUFFER_SIZE (buf) >= wav->blockalign) {
+        buf = gst_buffer_make_metadata_writable (buf);
+        GST_BUFFER_SIZE (buf) -= (GST_BUFFER_SIZE (buf) % wav->blockalign);
+      } else {
+        gst_buffer_unref (buf);
+        goto found_eos;
+      }
+    }
   }
 
   /* first chunk of data? create the source pad. We do this only here so
