@@ -134,7 +134,6 @@ gst_oss4_sink_base_init (gpointer g_class)
   templ = gst_pad_template_new ("sink", GST_PAD_SINK, GST_PAD_ALWAYS,
       gst_oss4_audio_get_template_caps ());
   gst_element_class_add_pad_template (element_class, templ);
-  gst_object_unref (templ);
 }
 
 static void
@@ -144,21 +143,21 @@ gst_oss4_sink_class_init (GstOss4SinkClass * klass)
   GstBaseSinkClass *basesink_class = (GstBaseSinkClass *) klass;
   GObjectClass *gobject_class = (GObjectClass *) klass;
 
-  gobject_class->dispose = gst_oss4_sink_dispose;
-  gobject_class->finalize = gst_oss4_sink_finalize;
-  gobject_class->get_property = gst_oss4_sink_get_property;
-  gobject_class->set_property = gst_oss4_sink_set_property;
+  gobject_class->dispose = GST_DEBUG_FUNCPTR (gst_oss4_sink_dispose);
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_oss4_sink_finalize);
+  gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_oss4_sink_get_property);
+  gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_oss4_sink_set_property);
 
   g_object_class_install_property (gobject_class, PROP_DEVICE,
       g_param_spec_string ("device", "Device",
           "OSS4 device (e.g. /dev/oss/hdaudio0/pcm0 or /dev/dspN) "
           "(NULL = use first available playback device)",
-          DEFAULT_DEVICE, G_PARAM_READWRITE | G_PARAM_STATIC_STRINGS));
+          DEFAULT_DEVICE, G_PARAM_READWRITE));
 
   g_object_class_install_property (gobject_class, PROP_DEVICE_NAME,
       g_param_spec_string ("device-name", "Device name",
           "Human-readable name of the sound device", DEFAULT_DEVICE_NAME,
-          G_PARAM_READABLE | G_PARAM_STATIC_STRINGS));
+          G_PARAM_READABLE));
 
   g_object_class_install_property (gobject_class,
       PROP_VOLUME,
@@ -392,7 +391,7 @@ gst_oss4_sink_getcaps (GstBaseSink * bsink)
   oss = GST_OSS4_SINK (bsink);
 
   if (oss->fd == -1) {
-    caps = gst_oss4_audio_get_template_caps ();
+    caps = gst_caps_copy (gst_oss4_audio_get_template_caps ());
   } else if (oss->probed_caps) {
     caps = gst_caps_copy (oss->probed_caps);
   } else {
