@@ -21,32 +21,23 @@
 
 /**
  * SECTION:element-waveformsink
- * @short_description: output sound using WaveForm API
  *
- * <refsect2>
- * <para>
- * This element lets you output sound using the WaveForm API.
- * </para>
- * <para>
+ * This element lets you output sound using the Windows WaveForm API.
+ *
  * Note that you should almost always use generic audio conversion elements
  * like audioconvert and audioresample in front of an audiosink to make sure
  * your pipeline works under all circumstances (those conversion elements will
  * act in passthrough-mode if no conversion is necessary).
- * </para>
+ *
+ * <refsect2>
  * <title>Example pipelines</title>
- * <para>
- * <programlisting>
- * gst-launch-0.10 -v audiotestsrc ! audioconvert ! volume volume=0.1 ! waveformsink
- * </programlisting>
- * will output a sine wave (continuous beep sound) to your sound card (with
+ * |[
+ * gst-launch -v audiotestsrc ! audioconvert ! volume volume=0.1 ! waveformsink
+ * ]| will output a sine wave (continuous beep sound) to your sound card (with
  * a very low volume as precaution).
- * </para>
- * <para>
- * <programlisting>
- * gst-launch-0.10 -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! waveformsink
- * </programlisting>
- * will play an Ogg/Vorbis audio file and output it.
- * </para>
+ * |[
+ * gst-launch -v filesrc location=music.ogg ! decodebin ! audioconvert ! audioresample ! waveformsink
+ * ]| will play an Ogg/Vorbis audio file and output it.
  * </refsect2>
  */
 
@@ -58,13 +49,6 @@
 
 GST_DEBUG_CATEGORY_STATIC (waveformsink_debug);
 
-/* elementfactory information */
-static const GstElementDetails gst_waveform_sink_details =
-GST_ELEMENT_DETAILS ("WaveForm Audio Sink",
-    "Sink/Audio",
-    "Output to a sound card via WaveForm API",
-    "Sebastien Moutte <sebastien@moutte.net>");
-
 static void gst_waveform_sink_base_init (gpointer g_class);
 static void gst_waveform_sink_class_init (GstWaveFormSinkClass * klass);
 static void gst_waveform_sink_init (GstWaveFormSink * wfsink,
@@ -122,7 +106,10 @@ gst_waveform_sink_base_init (gpointer g_class)
 {
   GstElementClass *element_class = GST_ELEMENT_CLASS (g_class);
 
-  gst_element_class_set_details (element_class, &gst_waveform_sink_details);
+  gst_element_class_set_details_simple (element_class, "WaveForm Audio Sink",
+      "Sink/Audio",
+      "Output to a sound card via WaveForm API",
+      "Sebastien Moutte <sebastien@moutte.net>");
   gst_element_class_add_pad_template (element_class,
       gst_static_pad_template_get (&waveformsink_sink_factory));
 }
@@ -435,8 +422,7 @@ gst_waveform_sink_unprepare (GstAudioSink * asink)
 
     for (index = 0; index < wfsink->buffer_count; index++) {
       if (wfsink->wave_buffers[index].dwFlags & WHDR_PREPARED) {
-        MMRESULT mmresult =
-            waveOutUnprepareHeader (wfsink->hwaveout,
+        MMRESULT mmresult = waveOutUnprepareHeader (wfsink->hwaveout,
             &wfsink->wave_buffers[index], sizeof (WAVEHDR));
         if (mmresult != MMSYSERR_NOERROR) {
           waveOutGetErrorText (mmresult, wfsink->error_string,
@@ -510,7 +496,7 @@ gst_waveform_sink_write (GstAudioSink * asink, gpointer data, guint length)
     memcpy (waveheader->lpData + waveheader->dwUser, data, bytes_to_write);
     waveheader->dwUser += bytes_to_write;
     remaining_length -= bytes_to_write;
-    data = (byte *) data + bytes_to_write;
+    data = (guint8 *) data + bytes_to_write;
 
     if (waveheader->dwUser == wfsink->buffer_size) {
       /* we have filled a buffer, let's prepare it and next write it to the device */
