@@ -40,7 +40,7 @@ video_crop_get_test_caps (GstElement * videocrop)
   GList *list = NULL;
   guint i;
 
-  srcpad = gst_element_get_pad (videocrop, "src");
+  srcpad = gst_element_get_static_pad (videocrop, "src");
   fail_unless (srcpad != NULL);
   allowed_caps = gst_pad_get_pad_template_caps (srcpad);
   fail_unless (allowed_caps != NULL);
@@ -208,6 +208,7 @@ videocrop_test_cropping_deinit_context (GstVideoCropTestContext * ctx)
   gst_buffer_replace (&ctx->last_buf, NULL);
   memset (ctx, 0x00, sizeof (GstVideoCropTestContext));
 }
+
 typedef void (*GstVideoCropTestBufferFunc) (GstBuffer * buffer);
 
 static void
@@ -254,23 +255,23 @@ check_1x1_buffer (GstBuffer * buf)
     /* the exact values we check for come from videotestsrc */
     switch (format) {
       case GST_MAKE_FOURCC ('I', '4', '2', '0'):
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[0], 76);
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[8], 85);
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[12], 255);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[0], 81);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[8], 90);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[12], 240);
         break;
       case GST_MAKE_FOURCC ('Y', 'V', '1', '2'):
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[0], 76);
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[8], 255);
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[12], 85);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[0], 81);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[8], 240);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[12], 90);
         break;
       case GST_MAKE_FOURCC ('Y', '8', '0', '0'):
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[0], 76);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[0], 81);
         /* no chroma planes */
         break;
       case GST_MAKE_FOURCC ('A', 'Y', 'U', 'V'):
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[1], 76);
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[2], 85);
-        fail_unless_equals_int (GST_BUFFER_DATA (buf)[3], 255);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[1], 81);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[2], 90);
+        fail_unless_equals_int (GST_BUFFER_DATA (buf)[3], 240);
         /* no chroma planes */
         break;
       default:
@@ -347,6 +348,11 @@ GST_START_TEST (test_crop_to_1x1)
     s = gst_caps_get_structure (caps, 0);
     fail_unless (s != NULL);
 
+    if (g_strcmp0 (gst_structure_get_name (s), "video/x-raw-gray") == 0) {
+      /* videotestsrc does not support this format */
+      continue;
+    }
+
     GST_INFO ("testing format: %" GST_PTR_FORMAT, caps);
 
     gst_structure_set (s, "width", G_TYPE_INT, 160,
@@ -368,8 +374,6 @@ GST_START_TEST (test_crop_to_1x1)
 
 GST_END_TEST;
 
-
-
 GST_START_TEST (test_cropping)
 {
   GstVideoCropTestContext ctx;
@@ -393,49 +397,58 @@ GST_START_TEST (test_cropping)
   videocrop_test_cropping_init_context (&ctx);
 
   caps_list = video_crop_get_test_caps (ctx.crop);
+  node = g_list_nth (caps_list, __i__);
 
-  for (node = caps_list; node != NULL; node = node->next) {
+  if (node != NULL) {
     GstStructure *s;
-    GstCaps *caps, *in_caps;
+    GstCaps *caps;
 
     caps = gst_caps_copy (GST_CAPS (node->data));
     s = gst_caps_get_structure (caps, 0);
     fail_unless (s != NULL);
 
-    GST_INFO ("testing format: %" GST_PTR_FORMAT, caps);
+    /* videotestsrc does not support this format, so ignore it */
+    if (!g_str_equal (gst_structure_get_name (s), "video/x-raw-gray")) {
+      GST_INFO ("testing format: %" GST_PTR_FORMAT, caps);
 
-    for (i = 0; i < G_N_ELEMENTS (sizes_to_try); ++i) {
-      GST_INFO (" - %d x %d", sizes_to_try[i].width, sizes_to_try[i].height);
+      for (i = 0; i < G_N_ELEMENTS (sizes_to_try); ++i) {
+        GstCaps *in_caps;
 
-      gst_structure_set (s, "width", G_TYPE_INT, sizes_to_try[i].width,
-          "height", G_TYPE_INT, sizes_to_try[i].height, NULL);
-      in_caps = gst_caps_copy (caps);
+        GST_INFO (" - %d x %d", sizes_to_try[i].width, sizes_to_try[i].height);
 
-      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 1, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 1, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 0, 1, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 1, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 63, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 63, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 0, 63, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 63, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 63, 0, 0, 1, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 63, 1, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 1, 63, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 1, 0, 0, 63, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 32, 0, 0, 128, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 32, 128, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 0, 128, 32, 0, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 128, 0, 0, 32, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 1, 1, 1, 1, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 63, 63, 63, 63, NULL);
-      videocrop_test_cropping (&ctx, in_caps, 64, 64, 64, 64, NULL);
+        gst_structure_set (s, "width", G_TYPE_INT, sizes_to_try[i].width,
+            "height", G_TYPE_INT, sizes_to_try[i].height, NULL);
+        in_caps = gst_caps_copy (caps);
 
-      gst_caps_unref (in_caps);
+        videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 1, 0, 0, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 1, 0, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 0, 1, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 1, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 63, 0, 0, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 63, 0, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 0, 63, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 63, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 63, 0, 0, 1, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 63, 1, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 1, 63, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 1, 0, 0, 63, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 0, 0, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 32, 0, 0, 128, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 32, 128, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 0, 128, 32, 0, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 128, 0, 0, 32, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 1, 1, 1, 1, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 63, 63, 63, 63, NULL);
+        videocrop_test_cropping (&ctx, in_caps, 64, 64, 64, 64, NULL);
+
+        gst_caps_unref (in_caps);
+      }
     }
+
     gst_caps_unref (caps);
+  } else {
+    GST_INFO ("no caps #%d", __i__);
   }
   g_list_foreach (caps_list, (GFunc) gst_caps_unref, NULL);
   g_list_free (caps_list);
@@ -464,7 +477,7 @@ GST_START_TEST (test_passthrough)
 
   g_object_set (ctx.src, "num-buffers", 1, NULL);
 
-  srcpad = gst_element_get_pad (ctx.src, "src");
+  srcpad = gst_element_get_static_pad (ctx.src, "src");
   fail_unless (srcpad != NULL);
   gst_pad_add_buffer_probe (srcpad, G_CALLBACK (buffer_probe_cb), &gen_buf);
   gst_object_unref (srcpad);
@@ -771,7 +784,7 @@ videocrop_suite (void)
   tcase_add_test (tc_chain, test_caps_transform);
   tcase_add_test (tc_chain, test_passthrough);
   tcase_add_test (tc_chain, test_unit_sizes);
-  tcase_add_test (tc_chain, test_cropping);
+  tcase_add_loop_test (tc_chain, test_cropping, 0, 25);
 
   return s;
 }
